=== modified file 'NEWS'
--- NEWS	2006-10-19 00:05:23 +0000
+++ NEWS	2006-10-19 03:25:55 +0000
@@ -92,6 +92,10 @@
 
    * diff3 does not indicate conflicts on clean merge. (Aaron Bentley)
 
+    * If a commit fails, the commit message is stored in a file at the root of
+      the tree for later commit. (Cheuksan Edward Wang, Stefan Metzmacher,
+      #32054)
+
   TESTING:
 
     * New test base class TestCaseWithMemoryTransport offers memory-only

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-10-18 12:54:37 +0000
+++ bzrlib/builtins.py	2006-10-19 03:25:55 +0000
@@ -23,6 +23,7 @@
 import codecs
 import errno
 import sys
+import tempfile
 
 import bzrlib
 from bzrlib import (
@@ -1801,24 +1802,78 @@
             reporter = ReportCommitToLog()
         else:
             reporter = NullCommitReporter()
-        
+
+        # save the commit message and only unlink it if the commit was
+        # successful
+        try:
+            tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr-commit-',
+                                                       dir=tree.basedir)
+            os.close(tmp_fileno)
+        except OSError:
+            try:
+                # No access to working dir, try $TMP
+                tmp_fileno, msgfilename= tempfile.mkstemp(prefix='bzr-commit-')
+                os.close(tmp_fileno)
+            except:
+                # We can't create a temp file, try to work without it
+                msgfilename = None
+        if msgfilename is not None:
+            msgfile = open(msgfilename, "wt")
+            try:
+                msgfile.write(message.encode(bzrlib.user_encoding, 'replace'))
+            finally:
+                msgfile.close()
+
         try:
             tree.commit(message, specific_files=selected_list,
                         allow_pointless=unchanged, strict=strict, local=local,
                         reporter=reporter)
+            if msgfilename is not None:
+                try:
+                    os.unlink(msgfilename)
+                except IOError, e:
+                    warning("failed to unlink %s: %s; ignored", msgfilename, e)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
-            raise errors.BzrCommandError("no changes to commit."
-                                         " use --unchanged to commit anyhow")
+            if msgfilename is not None:
+                raise errors.BzrCommandError("no changes to commit."
+                                  " use --unchanged to commit anyhow\n"
+                                  "Commit message saved. To reuse the message,"
+                                  " do\nbzr commit --file " + msgfilename)
+            else:
+                raise errors.BzrCommandError("no changes to commit."
+                                  " use --unchanged to commit anyhow")
         except ConflictsInTree:
-            raise errors.BzrCommandError("Conflicts detected in working tree.  "
-                'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+            if msgfilename is not None:
+                raise errors.BzrCommandError('Conflicts detected in working '
+                    'tree.  Use "bzr conflicts" to list, "bzr resolve FILE" to'
+                    ' resolve.\n'
+                    'Commit message saved. To reuse the message,'
+                    ' do\nbzr commit --file ' + msgfilename)
+            else:
+                raise errors.BzrCommandError('Conflicts detected in working '
+                    'tree.  Use "bzr conflicts" to list, "bzr resolve FILE" to'
+                    ' resolve.')
         except StrictCommitFailed:
-            raise errors.BzrCommandError("Commit refused because there are unknown "
-                                         "files in the working tree.")
+            if msgfilename is not None:
+                raise errors.BzrCommandError("Commit refused because there are"
+                                  " unknown files in the working tree.\n"
+                                  "Commit message saved. To reuse the message,"
+                                  " do\nbzr commit --file " + msgfilename)
+            else:
+                raise errors.BzrCommandError("Commit refused because there are"
+                                  " unknown files in the working tree.")
         except errors.BoundBranchOutOfDate, e:
-            raise errors.BzrCommandError(str(e) + "\n"
+            if msgfilename is not None:
+                raise errors.BzrCommandError(str(e) + "\n"
+                'To commit to master branch, run update and then commit.\n'
+                'You can also pass --local to commit to continue working '
+                'disconnected.\n'
+                'Commit message saved. To reuse the message,'
+                ' do\nbzr commit --file ' + msgfilename)
+            else:
+                raise errors.BzrCommandError(str(e) + "\n"
                 'To commit to master branch, run update and then commit.\n'
                 'You can also pass --local to commit to continue working '
                 'disconnected.')

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-10-19 03:25:55 +0000
@@ -35,10 +35,32 @@
         # If forced, it should succeed, but this is not tested here.
         self.run_bzr("init")
         self.build_tree(['hello.txt'])
-        result = self.run_bzr("commit", "-m", "empty", retcode=3)
-        self.assertEqual(('', 'bzr: ERROR: no changes to commit.'
-                              ' use --unchanged to commit anyhow\n'),
-                         result)
+        out,err = self.run_bzr("commit", "-m", "empty", retcode=3)
+        self.assertEqual('', out)
+        self.assertStartsWith(err, 'bzr: ERROR: no changes to commit.'
+                                  ' use --unchanged to commit anyhow\n')
+
+    def test_save_commit_message(self):
+        """Failed commit should save the message in a file"""
+        self.run_bzr("init")
+        out,err = self.run_bzr("commit", "-m", "message", retcode=3)
+        self.assertEqual('', out)
+        self.assertStartsWith(err, 'bzr: ERROR: no changes to commit.'
+                                  ' use --unchanged to commit anyhow\n'
+                                  'Commit message saved. To reuse the message,'
+                                  ' do\nbzr commit --file ')
+        message_file = re.compile('bzr-commit-\S*').search(err).group()
+        self.check_file_contents(message_file, 'message')
+
+    def test_commit_success(self):
+        """Successful commit should not leave behind a bzr-commit-* file"""
+        self.run_bzr("init")
+        self.run_bzr("commit", "--unchanged", "-m", "message")
+        self.assertEqual('', self.capture('unknowns'))
+
+        # same for unicode messages
+        self.run_bzr("commit", "--unchanged", "-m", u'foo\xb5')
+        self.assertEqual('', self.capture('unknowns'))
 
     def test_commit_with_path(self):
         """Commit tree with path of root specified"""

